import objectToFormData from 'object-to-formdata';
import applications from './manuals/applications';

function DialplanRules(core) {
    this.client = core.client;
    this.oldPath = '/v1/Dialplan/Rules';
}
const path = '/v1/PBX/Dialplans';
const oldApiUrl = 'dialplan/index.php';

const intermediateHandler = (client, options) => {
    return new Promise((resolve, reject) => {
        client(options)
            .then((res) => {
                resolve(res.data);
            })
            .catch((e) => {
                reject(e);
            });
    });
};

function CustomError(reason) {
    Error.call(this, reason);
    this.data = {
        reason
    };
    if (Error.captureStackTrace) {
        Error.captureStackTrace(this, CustomError);
    } else {
        this.stack = new Error().stack;
    }
}
CustomError.prototype = Object.create(Error.prototype);
CustomError.prototype.constructor = CustomError;

Object.assign(DialplanRules.prototype, {
    namespace: 'PBX.dialplans.rules',
    getList() {
        return this.client.send('GET', `${path}/?`);
    },
    getItem(params) {
        return this.client.send('GET', `${path}/${params.id}/`);
    },
    create(params) {
        return this.client.send('POST', `${path}/?`, params);
    },
    update(params) {
        return this.client.send('PUT', `${path}/${params.id}/`, params);
    },
    remove({id}) {
        return this.client.send('DELETE', `${path}/${id}/`);
    },
    export(params) {
        return this.client.send('POST', `${this.oldPath}/export`, params);
    },
    exportPing(params) {
        return this.client.send('GET', `${this.oldPath}/pingExport`, params);
    },
    import(data) {
        return intermediateHandler(this.client._transport, {
            url: oldApiUrl,
            method: 'post',
            data: objectToFormData(data),
            params: {
                'import_dialplan': '1'
            },
            transformResponse: [
                function(data) {
                    const response = JSON.parse(data);
                    if (response.error) {
                        throw new CustomError(response.error);
                    }
                    return {
                        type: 'result',
                        result: {
                            records: response
                        }
                    };
                }
            ]
        });
    },
    importConfirm(params) {
        return intermediateHandler(this.client._transport, {
            url: oldApiUrl,
            method: 'get',
            params,
            transformResponse: [
                function(data) {
                    const response = JSON.parse(data);
                    if (response.error) {
                        throw new CustomError(response.error);
                    }
                }
            ]
        });
    },
    getAdditionalRules() {
        return new Promise((resolve, reject) => {
            import(
                /*  webpackChunkName: 'chunks/additional_dialplans'  */
                './manuals/additional_dialplans.json'
            ).then((res) => {
                resolve({
                    type: 'result',
                    result: {
                        records: res.default
                    }
                });
            });
        });
    },
    getExtens() {
        return new Promise((resolve, reject) => {
            import(
                /*  webpackChunkName: 'chunks/json_extens'  */
                './manuals/json_extens.json'
            ).then((res) => {
                resolve({type: 'result', result: {extens: res.default}});
            });
        });
    },
    getAppsList() {
        return new Promise((resolve, reject) => {
            import(
                /*  webpackChunkName: 'chunks/appsList'  */
                './manuals/appsList.json'
            ).then((res) => {
                resolve({type: 'result', result: {appsList: res.default}});
            });
        });
    },
    getScheme() {
        return new Promise((resolve, reject) => {
            Promise.all(
                Object.entries(applications).map((item) => item[1])
            ).then((res) => {
                resolve({
                    type: 'result',
                    result: {records: res.map((item) => item.default)}
                });
            });
        });
    },
    getSelectFrom(params) {
        return intermediateHandler(this.client._transport, {
            url: oldApiUrl,
            method: 'get',
            params: {
                ...params,
                action: 'getSelectFrom',
                api: true
            },
            transformResponse: [
                function(data) {
                    const response = JSON.parse(data);
                    if (response.error) {
                        throw new CustomError(response.error);
                    }
                    return response;
                }
            ]
        });
    }
});

export default DialplanRules;
